<?php

namespace Drupal\mappi_lane;

use GuzzleHttp\ClientInterface;

class MappiApiClient {

  protected $httpClient;

  public function __construct(ClientInterface $httpClient) {
    $this->httpClient = $httpClient;
  }

  public function request(string $method, string $endpoint, array $options = []) {
    $response = $this->httpClient->request($method, $endpoint, $options);

    return json_decode($response->getBody()->getContents(), TRUE);
  }

  public function getMaps(string $baseUrl): array {
    $baseUrl = $this->normalBaseUrl($baseUrl);
    return $this->request('GET', 'https://' . $baseUrl . '/api/maps.json')['data'];
  }

  public function getMapSettings(string $baseUrl, $mapId): array {
    $mapId = (int) $mapId;
    $baseUrl = $this->normalBaseUrl($baseUrl);
    if ($mapId === 0) {
      return ['no categories found'];
    }
    return $this->request('GET', 'https://' . $baseUrl . '/api/maps/' . $mapId . '/settings.json')['data'];
  }

  protected function normalBaseUrl(string $baseUrl): string {
    return rtrim($baseUrl, '/');
  }

}
