<?php

namespace Drupal\mappi_lane\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\mappi_lane\MappiApiClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'mappi_embed' formatter.
 *
 * @FieldFormatter(
 *   id = "mappi_embed_formatter",
 *   label = @Translation("Mappi embed formatter"),
 *   field_types = {
 *     "mappi"
 *   }
 * )
 */
class MappiEmbedFormatter extends FormatterBase {
  protected MappiApiClient $mappiApiClient;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);

    $instance->setMappiApiClient($container->get('mappi_lane.api_client'));

    return $instance;
  }

  public function setMappiApiClient(MappiApiClient $mappiApiClient) {
    $this->mappiApiClient = $mappiApiClient;
  }

  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($items as $delta => $item) {
      $elements[$delta] = [
        '#theme' => 'mappi_embed',
        '#map_url' => $this->generateUrl($item),
      ];
    }

    return $elements;
  }

  protected function generateUrl($item) {
    $mapId = $item->value;
    $baseUrl = $this->getFieldSetting('customer_base_url');
    $mapSettings = $this->mappiApiClient->getMapSettings($baseUrl, $mapId);
    $categories = $this->getCategories($item, $mapSettings['categories']);
    $mapUrl = $mapSettings['url'];

    $settings = [
      "categories" => implode(',', $categories),
      "south" => $item->south ?? $mapSettings['bounds'][0][0],
      "east" => $item->east ?? $mapSettings['bounds'][0][1],
      "north" => $item->north ?? $mapSettings['bounds'][1][0],
      "west" => $item->west ?? $mapSettings['bounds'][1][1],
      "layer" => $item->layer ?? $mapSettings['layer'],
    ];

    return $mapUrl . '/#'
      . $settings['south'] . '/'
      . $settings['east'] . '/'
      . $settings['north'] . '/'
      . $settings['west'] . '/'
      . $settings['layer'] . '/'
      . $settings['categories'] . '//';
  }

  protected function getCategories($item, $mapCategories): array {
    $selectedCategories = json_decode($item->settings, TRUE)['categories'];
    if (!empty($selectedCategories)) {
      return $selectedCategories;
    }

    return $this->getDefaultCategories($mapCategories);
  }

  protected function getDefaultCategories($mapCategories): array {
    $defaultCategories = [];

    foreach ($mapCategories as $category) {
      if (!array_key_exists('children', $category)) {
        if ($category['isDefault']) {
          $defaultCategories[] = $category['id'];
        }
        continue;
      }

      foreach ($category['children'] as $child) {
        $defaultCategories[] = $child['id'];
      }
    }

    return $defaultCategories;
  }

}
