<?php

namespace Drupal\mappi_lane\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Provides a field type of mappi.
 *
 * @FieldType(
 *   id = "mappi",
 *   label = @Translation("Mappi field"),
 *   default_formatter = "mappi_default_formatter",
 *   default_widget = "mappi_default_widget",
 * )
 */
class MappiItem extends FieldItemBase {

  public static function defaultFieldSettings() {
    return ['customer_base_url' => 'demo.mappi.nl'];
  }

  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'varchar',
          'not null' => TRUE,
          'length' => 255,
        ],
        'settings' => [
          'type' => 'text',
          'size' => 'big',
        ],
      ],
    ];
  }

  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['customer_maps'] = DataDefinition::create('string')
      ->setLabel(t('Customer maps'));

    $properties['map_settings'] = DataDefinition::create('string')
      ->setLabel(t('Map settings'));

    return $properties;
  }

  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {

    $element['customer_base_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Base URL customer'),
      '#default_value' => $this->getSetting('customer_base_url'),
      '#placeholder' => ('kaart.katwijk.nl'),
      '#description' => $this->t('Used to call API for all available maps, dont include https and extension'),
    ];

    $element['#element_validate'] = [
      [static::class, 'fieldSettingsFormValidate'],
    ];

    return $element;
  }

}
